from functools import partial
from fasthtml.components import Div, Span
from lucide_fasthtml import Lucide
from .button import Button

def CollapsibleTrigger(*c, **kwargs):
    sr = Span("Toggle", cls="sr-only")
    return Button(Lucide("chevrons-up-down", cls="size-4"),sr, variant="ghost", size="icon", cls="w-9 p-0", data_ref="collapse-trigger", **kwargs)

def Collapsible(*c, state="closed", **kwargs):
    cls = f"{kwargs.pop('cls', '')} group"
    return Div(*c, data_ref="collapse",  data_state=state, cls=cls,  **kwargs)

def CollapsibleContent(*c, **kwargs):
    cls = f"{kwargs.pop('cls', '')} group-data-[state=closed]:hidden"
    return Div(*c, cls=cls,  data_ref="collapse-content", **kwargs)


from fasthtml.common import Ol, Nav, Li, A, Span
from fasthtml.common import Request
from lucide_fasthtml import Lucide

__all__ = ["Breadcrumb", "BreadcrumbItem", "BreadcrumbLink", "BreadcrumbSeparator", "BreadcrumbEllipsis", "BreadcrumbPage", "BreadcrumbList"]

breadcrumb_item_cls = "inline-flex items-center gap-1.5"
breadcrumb_link_cls = "transition-colors hover:text-foreground"
breadcrumb_list_cls = "flex flex-wrap items-center gap-1.5 break-words text-sm text-muted-foreground sm:gap-2.5"
breadcrumb_page_cls = "font-normal text-foreground"
breadcrumb_ellipsis_cls = "flex h-9 w-9 items-center justify-center"

def Breadcrumb(*c, req:Request=None, **kwargs):
    if req:
        path = req.url.path
        parts = path.strip('/').split('/')
        items = [(BreadcrumbItem(BreadcrumbLink("Home", href="/")), BreadcrumbSeparator())]
        for i, part in enumerate(parts):
            if i == len(parts) - 1:
                items.append(BreadcrumbItem(BreadcrumbPage(part.capitalize())))
                break
            else:
                link = '/' + '/'.join(parts[:i+1])
                items.append((BreadcrumbItem(BreadcrumbLink(part.capitalize(), href=link)), BreadcrumbSeparator()))

        if len(items) >= 4:
            items[1] = (BreadcrumbItem(BreadcrumbEllipsis()), BreadcrumbSeparator())
        return Nav(BreadcrumbList(*items), aria_label="breadcrumb", **kwargs)
    return Nav(*c, aria_label="breadcrumb", **kwargs)   

def BreadcrumbItem(*c, **kwargs):
    return Li(*c,cls=f"{kwargs.pop('cls', '')} {breadcrumb_item_cls}", **kwargs)

def BreadcrumbLink(*c, **kwargs):
    return A(*c, cls=f"{kwargs.pop('cls', '')} {breadcrumb_link_cls}", **kwargs)

def BreadcrumbEllipsis(**kwargs):
    return Span(Lucide(icon="ellipsis", cls="h-4 w-4"), Span("More", cls="sr-only"), cls=f"{kwargs.pop('cls', '')} {breadcrumb_ellipsis_cls}", **kwargs)

def BreadcrumbSeparator(icon="chevron-right", **kwargs):
    return Li(Lucide(icon=icon), role="presentation", aria_hidden="true", **kwargs)

def BreadcrumbList(*c, **kwargs):
    return Ol(*c, cls=f"{kwargs.pop('cls', '')} {breadcrumb_list_cls}", **kwargs)

def BreadcrumbPage(*c, **kwargs):
    return Span(*c, role="link", aria_disabled="true", aria_current="page", cls=f"{kwargs.pop('cls', '')} {breadcrumb_page_cls}", **kwargs)


from fasthtml.common import Div, Button
from typing import Literal
__all__ = ["Tooltip", "TooltipTrigger", "TooltipContent"]

tooltip_content_cls = "opacity-0 invisible absolute z-50 overflow-hidden rounded-md border bg-popover px-3 py-1.5 text-sm text-popover-foreground shadow-md transition-opacity duration-300"

def Tooltip(*c, method:str='hover', **kwargs):
    assert method in ['hover', 'click'], "Method must be either 'hover' or 'click'"
    return Div(*c, data_ref="tooltip", data_method=method, **kwargs)

def TooltipTrigger(c, **kwargs):
    if isinstance(c, str):
        return Button(c, data_ref="tooltip-trigger", **kwargs)
    return c(data_ref="tooltip-trigger", **kwargs)

def TooltipContent(*c, state='closed', **kwargs):
    return Div(*c, cls=f"{kwargs.get('cls', '')} {tooltip_content_cls}", data_state=state, data_ref="tooltip-content", **kwargs)



from fasthtml.common import Div, Ul, Li, A
from .button import Button
from lucide_fasthtml import Lucide

__all__ = ["Nav", "NavItem", "NavLink"]

nav_cls = "flex items-center space-x-4 lg:space-x-6"
nav_item_cls = "relative"
nav_link_cls = "text-sm font-medium transition-colors hover:text-primary"
nav_link_active_cls = "text-primary"

def Nav(*c, cls=None, **kwargs):
    new_cls = nav_cls
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Ul(*c, **kwargs)

def NavItem(*c, cls=None, **kwargs):
    new_cls = nav_item_cls
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Li(*c, **kwargs)

def NavLink(*c, href="#", active=False, cls=None, **kwargs):
    new_cls = nav_link_cls
    if active:
        new_cls += f" {nav_link_active_cls}"
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return A(*c, href=href, **kwargs)

def NavMenu(*c, cls=None, **kwargs):
    new_cls = "flex items-center"
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Div(*c, **kwargs)

def NavMenuTrigger(*c, cls=None, **kwargs):
    new_cls = "flex items-center space-x-1 text-sm font-medium"
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Button(*c, variant="ghost", size="sm", **kwargs)

def NavMenuContent(*c, cls=None, **kwargs):
    new_cls = "absolute top-full left-0 mt-2 w-48 rounded-md bg-white shadow-lg ring-1 ring-black ring-opacity-5 focus:outline-none"
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Div(*c, **kwargs)



from fasthtml.common import Div, ft_hx, H3
from lucide_fasthtml import Lucide

__all__ = ["Accordion", "AccordionItem", "AccordionTrigger", "AccordionContent"]

accordion_cls = "w-full"
accordion_item_cls = "border-b"
accordion_trigger_cls = "flex flex-1 items-center justify-between py-4 font-medium transition-all hover:underline"
accordion_content_cls = "hidden overflow-hidden text-sm transition-all peer-data-[state=closed]:animate-accordion-up peer-data-[state=open]:animate-accordion-down"
inner_content_cls = "pb-4 pt-0"

def Accordion(*c, cls=None, **kwargs):
    new_cls = accordion_cls
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Div(*c, data_ref="accordion", **kwargs)

def AccordionItem(*c, cls=None, **kwargs):
    new_cls = accordion_item_cls
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Div(*c, data_ref="accordion-item", **kwargs)

def AccordionTrigger(*c, cls=None, **kwargs):
    kwargs["cls"] = f"{cls} peer group flex"
    return H3(ft_hx('button',
            *c,
            Lucide(icon="chevron-down", cls="h-4 w-4 shrink-0 transition-transform duration-200 group-data-[state=open]:rotate-180"),
            type="button",
            cls=accordion_trigger_cls,
        ),
        data_ref="accordion-trigger",
        aria_expanded="false",
        data_state="closed",
        **kwargs
    )

def AccordionContent(*c, cls=None, **kwargs):
    new_cls = accordion_content_cls
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Div(Div(*c, cls=inner_content_cls), data_ref="accordion-content", role="region", **kwargs)
